/***************************************************************************
 *   Copyright (C) 2006-2007 Nicolas Hadacek <hadacek@kde.org>             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef CCSC_H
#define CCSC_H

#include "tools/base/tool_group.h"
#include "common/gui/pfile_ext.h"

namespace CCSC
{
//----------------------------------------------------------------------------
class Base : public Tool::Base
{
public:
  virtual TQString baseExecutable(bool, Tool::OutputExecutableType) const { return "ccsc"; }

private:
  virtual TQStringList checkExecutableOptions(bool withWine) const;
  virtual bool checkExecutableResult(bool withWine, TQStringList &lines) const;
  virtual PURL::Directory checkExecutableWorkingDirectory() const;
};

//----------------------------------------------------------------------------
class Group : public Tool::Group
{
public:
  Group();
  PURL::Url checkExecutableUrl() const { return _checkExecTmp.url(); }
  virtual TQString name() const { return "ccsc"; }
  virtual TQString label() const { return i18n("CCS Compiler"); }
  virtual TQString informationText() const;
  virtual Tool::Category checkDevicesCategory() const { return Tool::Category::Nb_Types; }
  virtual Tool::ExecutableType preferedExecutableType() const { return Tool::ExecutableType::Unix; }
  virtual Tool::CompileType compileType() const { return Tool::SingleFile; }
  virtual PURL::FileType implementationType(PURL::ToolType type) const { return (type==PURL::ToolType::Compiler ? PURL::CSource : PURL::Nb_FileTypes); }

private:
  Log::StringView _sview;
  PURL::TempFile  _checkExecTmp;

  virtual TQValueList<const Device::Data *> getSupportedDevices(const TQString &s) const;
  virtual Compile::Process *processFactory(const Compile::Data &data) const;
  virtual Compile::Config *configFactory(::Project *project) const;
  virtual BaseData baseFactory(Tool::Category) const;
  virtual Tool::SourceGenerator *sourceGeneratorFactory() const { /*TODO*/ return 0; }
  virtual VersionData getToolchainVersion();
};

} // namespace

#endif
